/********************************************************************/
/*                                                                  */
/*    PWMdtH[ (AiOeX^[ Rd)         */
/*                                                   for ATtiny13A  */
/*                                                       2021.08.16 */
/*                                                     (C)neko Java */
/********************************************************************/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>

#define SYSCLK	9600000UL			//ZtRCU9.6MHz
#define TEN_US	400000UL			// 4TCÑ[v10s(400kHz/4=10us)
#define LOOPCNT (SYSCLK / TEN_US)	// [vJEg

#define ADC_REP 5
#define ADC_THR	285		// 0.3V threshold value.
#define BZ_THR 110		// buzzer on threshold value.

#define DIFF1 20
#define DIFF2 5

/* pin definition ------------------------------ */
#define P_SW	PB1	// LOW level check (INT0) input.
#define P_ADC	PB2	// Vout feed back (ADC1) input.
#define P_VOUT	PB0	// Vout (OC0A) out.
#define P_BZ	PB4 // buzzer out.

#define BZ()	(PORTB & _BV(P_BZ))
#define BZ_H()	(PORTB |= _BV(P_BZ))
#define BZ_L()	(PORTB &= ~_BV(P_BZ))


//EGCg10us
static void wait_10us(uint8_t t){
  uint8_t cnt;
  while( t--)
  { for( cnt = 0; cnt<LOOPCNT; cnt++)
     asm volatile("nop");
  }
}
//EFCg1ms
static void wait_ms(uint16_t t){
  while(t--)
   wait_10us(100);
}

void init()
{
	cli();

	CLKPR = 0b10000000;						//VXeNbNύX
	CLKPR = 0;								//VXeNbN Ȃ 9.6MHz
	
	DDRB  = 0b00010001;
	PORTB = 0b11101000;

	MCUCR &= ~(1<<ISC01 | 1<<ISC00);	// INT0 Low level interrupt
	GIMSK |= (1<<INT0);					// enable INT0 interrupt

	TCCR0A = 0b10000011;				// 8bit high speed PWM
	TCCR0B = 0b00000001;				// 9.6MHz
	OCR0A = 0;
	GTCCR = 0b10000000;					// PWM start

	ADCSRA = 0;						// ADC disable.
	ADCSRB = 0b00000000;			//
	DIDR0  = 0b00000100;			// disable ADC1 pins digital input. 
	ADMUX  = 0b01000001;			// ADC1(PB2), 1.1Vref

	wait_ms(10);

	set_sleep_mode(SLEEP_MODE_PWR_DOWN);

	sei();
}

int adc_exec_pre()	//ADϊs
{
	int d;

	ADCSRA |= 0b01000000;			// ADC (start)
	loop_until_bit_is_set(ADCSRA, ADIF);

	d  = ADCW & 0x3FF;
	
	return d;
}

int adc_exec()	//ω
{
	int i;
	int d=0;
	
	for(i=0; i<ADC_REP; i++){
		d += adc_exec_pre();
		wait_10us(10);
	}
	return d/ADC_REP;
}

ISR(INT0_vect)
{
	int d=0;
	int e=0;

	wait_ms(10);

	ADCSRA  = 0b10000110;			// ADC enable,  Clock = 9.6MHz * 1/64

	d=adc_exec();		// ǂ
	
	do {
		d=adc_exec();

		if((d-ADC_THR < DIFF1 && d-ADC_THR > 0) ||
		   (ADC_THR-d < DIFF1 && ADC_THR-d > 0)){
			if(d>ADC_THR) ++e;
			if(d<ADC_THR) --e;
			if(e>DIFF2 || e<(DIFF2*(-1))){
				if(d<ADC_THR && OCR0A<255) ++OCR0A;
				if(d>ADC_THR && OCR0A>0) --OCR0A;
			}
		}else{

			if(d<ADC_THR && OCR0A<255) ++OCR0A;
			if(d>ADC_THR && OCR0A>0) --OCR0A;
			e=0;
		}
		if(OCR0A>BZ_THR){	// Buzzer ON. togle BZ pin.
			if(BZ()) BZ_L(); else BZ_H();
		}else if(BZ()) BZ_L();
		
	} while (OCR0A);

	ADCSRA  = 0;					// ADC disable.

	set_sleep_mode(SLEEP_MODE_PWR_DOWN);
	
}

int main (void)
{
	init();
	
	while(1)
	{
		sleep_mode();
	}
	
}
